<?php

namespace Drupal\syncart\Service;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\image\Entity\ImageStyle;
use Drupal\taxonomy\Entity\Term;
use Drupal\commerce_price\Price;

/**
 * Custom Cart Service.
 */
class SynRenderService implements SynRenderServiceInterface {

  /**
   * Провайдер корзины.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Constructs a new SynCartService object.
   */
  public function __construct(CartProviderInterface $cart_provider) {
    $this->cartProvider = $cart_provider;
    $this->currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    $this->currency_formatter = \Drupal::service('commerce_price.currency_formatter');
  }

  /**
   * Cart GET.
   */
  private function getCart($cid) {
    $cart = NULL;
    if ($cid) {
      $carts = $this->cartProvider->getCarts();
      if (count($carts)) {
        foreach ($carts as $key => $user_cart) {
          if ($user_cart->id() == $cid) {
            $cart = $user_cart;
          }
        }
      }
      else {
        $storage = \Drupal::entityTypeManager()->getStorage('commerce_order');
        $cart = $storage->load($cid);
      }
    }
    else {
      $cart = $this->cartProvider->getCart('default');
    }
    $this->cart = $cart;
    return $cart;
  }

  /**
   * Cart Info.
   */
  public function render($cid = FALSE, $render = []) {
    $cart = $this->getCart($cid);

    $qitems = 0;
    $render = [
      'qitems' => 0,
      'price' => 0,
      'number' => 0,
      'quantity' => 0,
    ];
    if ($cart) {
      /** @var \Drupal\commerce_order\OrderStorageInterface $cart */
      $promotionsSum = 0;
      $items = $cart->getItems();
      $render['id'] = $cart->id();
      $unsortedItems = [];
      $itemsData = [];
      foreach ($items as $index => $orderItem) {
        $qitems++;
        $variationInfo = $this->getVariationInfo($orderItem);
        if (empty($variationInfo)) {
          continue;
        }
        if (!empty($variationInfo['data'])) {
          $itemsData[] = $variationInfo['id'];
        }
        $unsortedItems[$orderItem->id()] = $variationInfo;
        // $render['items'][$orderItem->id()] = $variationInfo;
        $render['quantity'] += $orderItem->getQuantity();
        if (!empty($variationInfo['adjustments'])) {
          foreach ($variationInfo['adjustments'] as $adjustment) {
            if ($adjustment['type'] == 'promotion') {
              $promotionsSum += $adjustment['number'];
            }
          }
        }
      }
      $groups = [];
      $sortedItems = [];
      if (!empty($itemsData)) {
        foreach ($itemsData as $group) {
          $groups[] = $group;
          $unsortedItems[$group]['group'] = $group;
          $sortedItems[] = $unsortedItems[$group];
          foreach ($unsortedItems[$group]['data'][0] as $key => $quantity) {
            $tempData = explode(':', $key);
            if ($tempData[0] == 'promotion') {
              $promotion = \Drupal::entityTypeManager()->getStorage('commerce_promotion')->load($tempData[1]);
              $configuration = $promotion->getOffer()->getConfiguration();
              foreach ($configuration['buy_conditions'][0]['configuration']['products'] as $productArray) {
                $products = \Drupal::entityTypeManager()->getStorage('commerce_product')->loadByProperties(['uuid' => $productArray['product']]);
                $product = array_keys($products);
                $productId = array_shift($product);
                foreach ($unsortedItems as $itemId => $item) {
                  if ($item['pid'] == $productId) {
                    $item['group'] = $group;
                    $sortedItems[] = $item;
                    unset($unsortedItems[$item['id']]);
                  }
                }
              }
            }
          }
          unset($unsortedItems[$group]);
        }
        $sortedItems = array_reverse($sortedItems);
      }
      foreach ($unsortedItems as $item) {
        $groups[] = $item['group'];
        $sortedItems[] = $item;
      }
      $render['groups'] = $groups;
      $render['items'] = $sortedItems;
      $adj_sum = 0;
      foreach ($cart->getAdjustments() as $index => $adjustment) {
        $adj_sum += $adjustment->getAmount()->getNumber();
        $adj_price = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $adjustment->getAmount(),
          ],
        ];
        $adj_price_renderable = \Drupal::service('renderer')->renderRoot($adj_price);
        $render['adjustments'][$index] = [
          'label' => $adjustment->getLabel(),
          'type' => $adjustment->getType(),
          'source' => $adjustment->getSourceId(),
          'is_included' => $adjustment->isIncluded(),
          'price' => $adj_price_renderable,
        ];
      }
      if (!empty($items)) {
        $total = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $cart->getTotalPrice(),
          ],
        ];
        $subtotal = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $cart->getSubtotalPrice(),
          ],
        ];
        $price_sum = 0;
        foreach ($cart->getItems() as $item) {
          $variation = $item->getPurchasedEntity();
          $price_sum += $variation->getPrice()->getNumber() * $item->getQuantity();
        };
        $full_price = new Price($price_sum, $cart->getSubtotalPrice()->getCurrencyCode());
        $full_price = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $full_price,
          ],
        ];
        $render['promotion'] = $promotionsSum;
        $promotion = new Price($promotionsSum, $cart->getSubtotalPrice()->getCurrencyCode());
        $promotion = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $promotion,
          ],
        ];
        $promotion_renderable = \Drupal::service('renderer')->renderRoot($promotion);
        $render['promosum'] = $promotion_renderable;
        $total_renderable = \Drupal::service('renderer')->renderRoot($total);
        $full_price_renderable = \Drupal::service('renderer')->renderRoot($full_price);
        $subtotal_renderable = \Drupal::service('renderer')->renderRoot($subtotal);
        $render['qitems'] = $qitems;
        $render['price'] = $subtotal_renderable;
        $render['subtotal'] = $subtotal_renderable;
        $render['total'] = $total_renderable;
        $render['full_price'] = $full_price_renderable;
        $render['number'] = $cart->getSubtotalPrice()->getNumber();
      }
    }
    return $render;
  }

  /**
   * Получить информацию о вариации из товара в корзине.
   */
  private function getVariationInfo($orderItem, $info = []) {
    if (is_object($variation = $orderItem->getPurchasedEntity())) {
      $this->getVariationProductInfo($variation, $info);
      $currencyCode = $variation->getPrice()->getCurrencyCode();
      $config = \Drupal::config('syncart.settings');
      $checkstore = (bool) $config->get('check_store') ?? FALSE;
      $info['checkstore'] = $checkstore;
      $stock = FALSE;
      if ($variation->field_stock && $checkstore) {
        $stock = intval($variation->field_stock->value);
      }
      $info['id'] = $orderItem->id();
      $info['group'] = $orderItem->id();
      if (!empty($adjustments = $orderItem->getAdjustments())) {
        foreach ($adjustments as $key => $adjustment) {
          $adjustment_price = $adjustment->getAmount()->getNumber();
          $currency_code = $adjustment->getAmount()->getCurrencyCode();
          $currency = $this->currency_storage->load($currency_code);
          $info['adjustments'][] = [
            'type' => $adjustment->getType(),
            'label' => $adjustment->getLabel(),
            'number' => $adjustment->getAmount()->getNumber(),
            'amount' => $this->currency_formatter->format($adjustment_price, $currency_code, []),
            'currencyCode' => $currency_code,
            'currencySymbol' => $currency->getSymbol(),
            'percentage' => $adjustment->getPercentage(),
          ];
        }
      }
      // dsm($orderItem->data->getValue());
      $info['data'] = $orderItem->data->getValue();
      if ($orderItem->hasField('field_custom')) {
        $info['custom'] = $orderItem->field_custom->value;
      }
      $info['quantity'] = round($orderItem->getQuantity(), 2);
      $info['stock'] = $stock;
      $info['number'] = $variation->getPrice()->getNumber();
      $price = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $variation->getPrice(),
        ],
      ];
      $total = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $orderItem->getTotalPrice(),
        ],
      ];
      $info['price'] = \Drupal::service('renderer')->renderRoot($price);
      $info['totalPrice'] = \Drupal::service('renderer')->renderRoot($total);
      \Drupal::moduleHandler()->alter('syncart_variation', $info, $variation);
    }
    return $info;
  }

  /**
   * Получить информацию о товаре вариации.
   */
  private function getVariationProductInfo($variation, &$info) {
    if (is_object($product = $variation->getProduct())) {
      if ($product->field_title && $product->field_title->value) {
        $title = $product->field_title->value;
      }
      if ($variation->field_variation_image) {
        $variationImage = $variation->field_variation_image->entity;
        if (is_object($variationImage)) {
          $uri = $variationImage->getFileUri();
          $image = ImageStyle::load('product_cart')->buildUrl($uri);
        }
      }

      $attributes = [];
      $attr = $variation->getAttributeValues();
      if ($attr) {
        foreach ($attr as $key => $value) {
          $attribute_item[] = [
            'id' => $value->id(),
            'label' => $value->getAttribute()->label(),
            'name' => $value->getName(),
          ];
        }
        if (!in_array($attribute_item, $attributes)) {
          $attributes[] = $attribute_item;
        }
      }

      $info['attributes'] = $attributes;
      $info['title'] = $title ?? $product->getTitle();
      if ($product->hasField('field_article')) {
        $article = $product->field_article->value;
      }
      $info['article'] = $article ?? '';
      $info['picture'] = $image ?? $this->getProductPicture($product);
      $info['url'] = $product->toUrl()->toString();
      $info['pid'] = $product->id();
      $info['body'] = $product->body->value ?? '';
      $info['vid'] = $variation->id();
      $uname = '';
      $qstep = 1;
      if ($product->hasField('field_unit')) {
        $unit = $product->get('field_unit')->getValue();
        if (!empty($unit)) {
          $term = Term::load($unit[0]['target_id']);
          $uname = $term->getName();
          if (!empty($term->field_unit_float->value) && (int) $term->field_unit_float->value != 0) {
            $config = \Drupal::config('core.entity_form_display.commerce_order_item.default.default');
            $qstep = round($config->getRawData()['content']['quantity']['settings']['step'], 2);
          }
        }
      }
      if ($product->hasField('field_package')) {
        $qstep = $product->field_package->value;
      }
      $info['qstep'] = (float) $qstep;
      $info['uname'] = $uname;
    }
  }

  /**
   * Загрузка изображения товара.
   */
  private function getProductPicture($product) {
    $image = '';
    $imageUri = '';
    $fieldImage = $product->field_image;
    $fieldGallery = $product->field_gallery;
    if ($fieldImage && !$fieldImage->isEmpty()) {
      foreach ($fieldImage as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    elseif ($fieldGallery && !$fieldGallery->isEmpty()) {
      foreach ($fieldGallery as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    if ($imageUri) {
      $image = ImageStyle::load('product_cart')->buildUrl($imageUri);
    }
    return $image;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $items = [];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
    }
    return empty($items);
  }

}
